﻿using System;

using DevExpress.XtraMap;

namespace TestProject
{
    /// <summary>
    /// 지오코드 데이터
    /// </summary>
    public class GeocodeData : IInformationData
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 데이터 응답시 이벤트 - OnDataResponse

        /// <summary>
        /// 데이터 응답시 이벤트
        /// </summary>
        public event EventHandler<RequestCompletedEventArgs> OnDataResponse;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 위치 정보
        /// </summary>
        private LocationInformation locationInformation = new LocationInformation();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 위치 정보 - LocationInformation

        /// <summary>
        /// 위치 정보
        /// </summary>
        public LocationInformation LocationInformation
        {
            get
            {
                return this.locationInformation;
            }
            set
            {
                this.locationInformation = value;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 주소 계산하기 - CalculateAddress(geoPoint)

        /// <summary>
        /// 주소 계산하기
        /// </summary>
        /// <param name="geoPoint">지리 포인트</param>
        public void CalculateAddress(GeoPoint geoPoint)
        {
            string addressString = "Address from your service here " + Environment.NewLine + "Coordinates: " + geoPoint.ToString();

            LocationInformation locationInformation = new LocationInformation();

            locationInformation.Address  = new Address(addressString);
            locationInformation.Location = new GeoPoint(geoPoint.Latitude, geoPoint.Longitude);

            LocationInformation = locationInformation;

            FireOnDataResponseEvent();
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 데이터 수신시 이벤트 발생시키기 - FireOnDataResponseEvent()

        /// <summary>
        /// 데이터 수신시 이벤트 발생시키기
        /// </summary>
        protected void FireOnDataResponseEvent()
        {
            OnDataResponse?.Invoke(this, CreateEventArgs());
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 이벤트 인자 생성하기 - CreateEventArgs()

        /// <summary>
        /// 이벤트 인자 생성하기
        /// </summary>
        /// <returns>요청 완료시 이벤트 인자</returns>
        private RequestCompletedEventArgs CreateEventArgs()
        {
            MapItem item = new MapCallout()
            {
                Location = this.locationInformation.Location,
                Text     = this.locationInformation.Address.FormattedAddress
            };

            return new RequestCompletedEventArgs
            (
                new MapItem[] { item },
                null,
                false,
                null
            );
        }

        #endregion
    }
}